@@ -1,11 +1,10 @@
 /*
- *  $Id: checksum.c,v 1.1.1.1 1998/10/16 17:53:08 route Exp $
+ *  $Id: checksum.c,v 1.6 1998/12/09 20:25:51 route Exp $
  *
  *  libnet
- *  checksum implementation
- *  checksum.c - IP-type checksum routines
+ *  checksum.c - IP checksum routines
  *
- *  Copyright (c) 1998 route|daemon9 <route@infonexus.com>
+ *  Copyright (c) 1998, 1999 route|daemon9 <route@infonexus.com>
  *  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -37,6 +36,7 @@
 int
 do_checksum(u_char *buf, int protocol, int len)
 {
+    struct icmpheader *icmp_hdr;
     struct tcphdr    *tcp_hdr;
     struct udphdr    *udp_hdr;
     struct ip        *ip_hdr;
@@ -48,6 +48,10 @@ do_checksum(u_char *buf, int protocol, int len)
         case IPPROTO_TCP:
             ip_hdr  = (struct ip *)(buf);
             tcp_hdr = (struct tcphdr *)(buf + IP_H);
+#if (SOLARIS_CKSUM_BUG)
+            tcp_hdr->th_sum = TCP_H;
+            break;
+#endif
             tcp_hdr->th_sum = 0;
 #if (__i386__)
             tcp_hdr->th_sum = tcp_check(tcp_hdr,
@@ -79,13 +83,6 @@ do_checksum(u_char *buf, int protocol, int len)
             free(p);
             p = NULL;
 #endif /* __i386__ */
-#if (__svr4__)
-            /*
-             *  Needed to make Solaris checksums work.  What kind of
-             *  retardedness is this, anyway?
-             */
-            ip_hdr->ip_sum = len + TCP_H;
-#endif /* __svr4__ */
             break;
         case IPPROTO_UDP:
             ip_hdr  = (struct ip *)(buf);
@@ -115,8 +112,18 @@ do_checksum(u_char *buf, int protocol, int len)
             free(p);
             p = NULL;
             break;
+        case IPPROTO_ICMP:
+            icmp_hdr = (struct icmpheader *)(buf + IP_H); 
+            icmp_hdr->icmp_sum = 0;
+            icmp_hdr->icmp_sum = ip_check((u_short *)(icmp_hdr), len);
+            break;
+        case IPPROTO_IP:
+            ip_hdr = (struct ip *)(buf);
+            ip_hdr->ip_sum = 0;
+            ip_hdr->ip_sum = ip_check((u_short *)buf, len);
+            break;
         default:
-            DEBUG("do_checksum: UNSUP protocol\n");
+            DEBUG("do_checksum: UNSUPPORTED protocol %d\n", protocol);
             break;
     }
     return (1);
